@@ -2,17 +2,52 @@
 ***********
 Annotations
 ***********
-TODO_Sphinx (I'm not sure about this comment block)
 
-Annotations are standard dictionary objects
-   
-~~~~~~~~~~~~~~
-Helper Methods
-~~~~~~~~~~~~~~
+Annotations are arbitrary metadata attached to Synapse entities. They can be
+accessed like ordinary object properties or like dictionary keys::
 
-.. automethod:: synapseclient.annotations.is_synapse_annotations
-.. automethod:: synapseclient.annotations.to_synapse_annotations
-.. automethod:: synapseclient.annotations.from_synapse_annotations
+    entity.my_annotation = 'This is one way to do it'
+    entity['other_annotation'] = 'This is another'
+
+Annotations can be given in the constructor for Synapse Entities::
+
+    entity = File('data.xyz', parent=my_project, rating=9.1234)
+
+Annotate the entity with location data::
+
+    entity.lat_long = [47.627477, -122.332154]
+
+Record when we collected the data::
+
+    from datetime import datetime as Datetime
+    entity.collection_date = Datetime.now()
+
+See:
+
+- :py:meth:`synapseclient.Synapse.getAnnotation`
+- :py:meth:`synapseclient.Synapse.setAnnotation`
+
+~~~~~~~~~~~~~~~~~~~~~~~
+Annotating data sources
+~~~~~~~~~~~~~~~~~~~~~~~
+
+Data sources are best recorded using Synapse's `provenance <Activity.html>`_ tools.
+
+~~~~~~~~~~~~~~~~~~~~~~
+Implementation details
+~~~~~~~~~~~~~~~~~~~~~~
+
+In Synapse, entities have both properties and annotations. Properties are used by
+the system, whereas annotations are completely user defined. In the Python client,
+we try to present this situation as a normal object, with one set of properties.
+
+For more on the implementation and a few gotchas, see the documentation on
+:py:mod:`synapseclient.entity`.
+
+See also:
+
+- :py:class:`synapseclient.entity.Entity`
+- :py:mod:`synapseclient.entity`
 
 """
 
@@ -66,7 +101,7 @@ def from_synapse_annotations(annotations):
     for key, value in annotations.iteritems():
         if key=='dateAnnotations':
             for k,v in value.iteritems():
-                #debug: print "%s=>%s\n" % (k,v,)
+                # print "%s=>%s\n" % (k,v,)
                 annos.setdefault(k,[]).extend([from_unix_epoch_time(float(t)) for t in v])
         elif key in ['stringAnnotations','longAnnotations','doubleAnnotations']:
             for k,v in value.iteritems():
