@@ -3,55 +3,21 @@
 Entity
 ******
 
-The Entity class is the base class for all entities. It has a few
-special characteristics. It is a dictionary-like object in which
-both object and dictionary notation (entity.foo or entity['foo'])
-can be used interchangeably.
+The Entity class is the base class for all entities, including Project, Folder
+and File, as well as deprecated entity types such as Data, Study, Summary,
+etc.
 
-In Synapse, entities have both properties and annotations. This has
-come to be viewed as awkward, so we try to hide it. Furthermore,
-because we're getting tricky with the dot notation, there are three
-distinct namespaces to consider when accessing variables that are
-part of the entity: the members of the object, properties defined by
-Synapse, and Synapse annotations, which are open-ended and user-
-defined.
+Entities are dictionary-like objects in which both object and dictionary
+notation (entity.foo or entity['foo']) can be used interchangeably.
 
-The rule, for either getting or setting is: first look in the object
-then look in properties, then look in annotations. If the key is not
-found in any of these three, a get results in a ``KeyError`` and a set
-results in a new annotation being created. Thus, the following results
-in a new annotation that will be persisted in Synapse::
+Imports::
 
-    entity.foo = 'bar'
-
-To create an object member variable, which will *not* be persisted in
-Synapse, this unfortunate notation is required::
-
-    entity.__dict__['foo'] = 'bar'
-
-Between the three namespaces, name collisions are entirely possible.
-Keys in the three namespaces can be referred to unambiguously like so::
-
-    entity.__dict__['key']
+    from synapseclient import Project, Folder, File
     
-    entity.properties.key
-    entity.properties['key']
-    
-    entity.annotations.key
-    entity.annotations['key']
-
-Alternate implementations include:
-- a naming convention to tag object members
-- keeping a list of 'transient' variables (the object members)
-- giving up on the dot notation (implemented in Entity2.py in commit e441fcf5a6963118bcf2b5286c67fc66c004f2b5 in the entity_object branch)
-- giving up on hiding the difference between properties and annotations
+    ## the following Entity types are deprecated
+    from synapseclient import Analysis, Code, Data, Study, Summary
 
 .. autoclass:: synapseclient.entity.Entity
-   :members:
-   
-.. automethod:: synapseclient.entity.is_locationable
-.. automethod:: synapseclient.entity.is_versionable
-.. automethod:: synapseclient.entity.split_entity_namespaces
    
 ~~~~~~~
 Project
@@ -71,31 +37,109 @@ File
 
 .. autoclass:: synapseclient.entity.File
 
+
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+Properties and annotations, implementation details
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+In Synapse, entities have both properties and annotations. Properties are used
+by the system, whereas annotations are completely user defined. In the Python
+client, we try to present this situation as a normal object, with one set of
+properties.
+
+Printing an entity will show the division between properties and annotations.::
+
+    print entity
+
+Under the covers, an Entity object has two dictionaries, one for properties and one
+for annotations. These two namespaces are distinct, so there is a possibility of
+collisions. It is recommended to avoid defining annotations with names that collide
+with properties, but this is not enforced.::
+
+    ## don't do this!
+    entity.properties['description'] = 'One thing'
+    entity.annotations['description'] = 'A different thing'
+
+In case of conflict, properties will take precedence.::
+
+    print entity.description
+    #> One thing
+
+Some additional ambiguity is entailed in the use of dot notation. Entity
+objects have their own internal properties which are not persisted to Synapse.
+As in all Python objects, these properties are held in object.__dict__. For
+example, this dictionary holds the keys 'properties' and 'annotations' whose
+values are both dictionaries themselves.
+
+The rule, for either getting or setting is: first look in the object then look
+in properties, then look in annotations. If the key is not found in any of
+these three, a get results in a ``KeyError`` and a set results in a new
+annotation being created. Thus, the following results in a new annotation that
+will be persisted in Synapse::
+
+    entity.foo = 'bar'
+
+To create an object member variable, which will *not* be persisted in
+Synapse, this unfortunate notation is required::
+
+    entity.__dict__['foo'] = 'bar'
+
+As mentioned previously, name collisions are entirely possible.
+Keys in the three namespaces can be referred to unambiguously like so::
+
+    entity.__dict__['key']
+
+    entity.properties.key
+    entity.properties['key']
+
+    entity.annotations.key
+    entity.annotations['key']
+
+Most of the time, users should be able to ignore these distinctions and treat
+Entities like normal Python objects. End users should never need to manipulate
+items in __dict__.
+
+See also:
+
+- :py:mod:`synapseclient.annotations`
+
 """
 
 import collections
 import itertools
 
 from synapseclient.dict_object import DictObject
-from synapseclient.utils import id_of, class_of, itersubclasses
+import synapseclient.utils as utils
+from synapseclient.utils import id_of, itersubclasses
 from synapseclient.exceptions import *
 import os
 
 
 # File, Locationable and Summary are Versionable
 class Versionable(object):
-    """TODO_Sphinx."""
+    """An entity for which Synapse will store a version history."""
     
     _synapse_entity_type = 'org.sagebionetworks.repo.model.Versionable'
     _property_keys = ['versionNumber', 'versionLabel', 'versionComment', 'versionUrl', 'versions']
 
+
 ## TODO: inherit from UserDict.DictMixin?
 ##       http://docs.python.org/2/library/userdict.html#UserDict.DictMixin
+
+## Alternate implementations include:
+## - a naming convention to tag object members
+## - keeping a list of 'transient' variables (the object members)
+## - giving up on the dot notation (implemented in Entity2.py in commit e441fcf5a6963118bcf2b5286c67fc66c004f2b5 in the entity_object branch)
+## - giving up on hiding the difference between properties and annotations
+
 class Entity(collections.MutableMapping):
     """
     A Synapse entity is an object that has metadata, access control, and
     potentially a file. It can represent data, source code, or a folder
     that contains other entities.
+
+    Entities should typically be created using the constructors for specific
+    subclasses such as Project, Folder or File.
     """
 
     _synapse_entity_type = 'org.sagebionetworks.repo.model.Entity'
@@ -111,7 +155,7 @@ class Entity(collections.MutableMapping):
         Create an Entity or a subclass given dictionaries of properties
         and annotations, as might be received from the Synapse Repository.
 
-        :param properties:  TODO_Sphinx
+        :param properties:  A map of Synapse properties
         
             If 'concreteType' is defined in properties, we create the proper subclass
             of Entity. If not, give back the type whose constructor was called:
@@ -119,7 +163,7 @@ class Entity(collections.MutableMapping):
             If passed an Entity as input, create a new Entity using the input
             entity as a prototype.
             
-        :param annotations: TODO_Sphinx
+        :param annotations: A map of user defined annotations
         :param local_state: Allow local state to be given.  
                             This state information is not persisted 
                             in the Synapse Repository.
@@ -127,18 +171,19 @@ class Entity(collections.MutableMapping):
         
         # Create a new Entity using an existing Entity as a prototype
         if isinstance(properties, Entity):
-            annotations = properties.annotations + (annotations if annotations else {})
-            local_state = properties.local_state() + (local_state if local_state else {})
+            if annotations is None: annotations = {}
+            if local_state is None: local_state = {}
+            annotations.update(properties.annotations)
+            local_state.update(properties.local_state())
             properties = properties.properties
-            del properties['id']
+            if 'id' in properties: del properties['id']
+            
         if cls==Entity and 'concreteType' in properties and properties['concreteType'] in _entity_type_to_class:
             cls = _entity_type_to_class[properties['concreteType']]
         return cls(properties=properties, annotations=annotations, local_state=local_state)
 
     @classmethod
     def getURI(self, id):
-        """TODO_Sphinx."""
-        
         return '/entity/%s' %id
 
 
@@ -203,18 +248,12 @@ class Entity(collections.MutableMapping):
 
 
     def postURI(self):
-        """TODO_Sphinx."""
-        
         return '/entity'
 
     def putURI(self):
-        """TODO_Sphinx."""
-        
         return '/entity/%s' %self.id
 
     def deleteURI(self):
-        """TODO_Sphinx."""
-        
         return '/entity/%s' %self.id
 
 
@@ -236,14 +275,10 @@ class Entity(collections.MutableMapping):
 
 
     def __setattr__(self, key, value):
-        """TODO_Sphinx."""
-        
         return self.__setitem__(key, value)
 
 
     def __setitem__(self, key, value):
-        """TODO_Sphinx."""
-        
         if key in self.__dict__:
             # If we assign like so:
             #   entity.annotations = {'foo';123, 'bar':'bat'}
@@ -261,16 +296,12 @@ class Entity(collections.MutableMapping):
     ## TODO: def __delattr__
 
     def __getattr__(self, key):
-        """TODO_Sphinx."""
-        
         # Note: that __getattr__ is only called after an attempt to
         # look the key up in the object's dictionary has failed.
         return self.__getitem__(key)
 
 
     def __getitem__(self, key):
-        """TODO_Sphinx."""
-        
         if key in self.__dict__:
             return self.__dict__[key]
         elif key in self.properties:
@@ -281,8 +312,6 @@ class Entity(collections.MutableMapping):
             raise KeyError(key)
 
     def __delitem__(self, key):
-        """TODO_Sphinx."""
-        
         if key in self.properties:
             del self.properties[key]
         elif key in self.annotations:
@@ -290,17 +319,14 @@ class Entity(collections.MutableMapping):
 
 
     def __iter__(self):
-        """TODO_Sphinx."""
-        
         return iter(self.keys())
 
 
     def __len__(self):
-        """TODO_Sphinx."""
-        
         return len(self.keys())
 
 
+    ## TODO shouldn't these include local_state as well? -jcb
     def keys(self):
         """Returns a set of property and annotation keys"""
         
@@ -312,8 +338,6 @@ class Entity(collections.MutableMapping):
         return key in self.properties or key in self.annotations
 
     def __str__(self):
-        """TODO_Sphinx."""
-        
         from cStringIO import StringIO
         f = StringIO()
 
@@ -403,6 +427,12 @@ class File(Entity, Versionable):
 
     When a File object is stored, the associated local file or its URL will be
     stored in Synapse. A File must have a path (or URL) and a parent.
+    
+    :param path:         Location to be represented by this File
+    :param name:         Name of the file in Synapse, not to be confused with the name within the path
+    :param parent:       Project or Folder where this File is stored
+    :param synapseStore: Whether the File should be uploaded or if only the path should be stored.
+                         Defaults to True (file should be uploaded)
 
     ::
     
@@ -418,7 +448,7 @@ class File(Entity, Versionable):
     def __init__(self, path=None, parent=None, synapseStore=True, properties=None, 
                  annotations=None, local_state=None, **kwargs):
         if path and 'name' not in kwargs:
-            kwargs['name'] = os.path.basename(path)
+            kwargs['name'] = utils.guess_file_name(path)
         self.__dict__['path'] = path
         if path:
             cacheDir, basename = os.path.split(path)
@@ -449,16 +479,19 @@ class Analysis(Entity):
 
 class Code(Entity, Locationable):
     _synapse_entity_type = 'org.sagebionetworks.repo.model.Code'
+    _local_keys = Locationable._local_keys
     _property_keys = Entity._property_keys + Locationable._property_keys
 
 
 class Data(Entity, Locationable):
     _synapse_entity_type = 'org.sagebionetworks.repo.model.Data'
+    _local_keys = Locationable._local_keys
     _property_keys = Entity._property_keys + Locationable._property_keys
 
 
 class Study(Entity, Locationable):
     _synapse_entity_type = 'org.sagebionetworks.repo.model.Study'
+    _local_keys = Locationable._local_keys
     _property_keys = Entity._property_keys + Locationable._property_keys
 
 
@@ -489,7 +522,7 @@ def split_entity_namespaces(entity):
         return (entity.properties.copy(), entity.annotations.copy(), entity.local_state())
 
     if not isinstance(entity, collections.Mapping):
-        raise SynapseMalformedEntityError("Can't call split_entity_namespaces on objects of type: %s" % class_of(entity))
+        raise SynapseMalformedEntityError("Can't split a %s object." % entity.__class__.__name__)
 
     if 'concreteType' in entity and entity['concreteType'] in _entity_type_to_class:
         entity_class = _entity_type_to_class[entity['concreteType']]
@@ -539,8 +572,6 @@ ENTITY_TYPES = LOCATIONABLE_TYPES + [
 ]
 
 def is_synapse_entity(entity):
-    """TODO_Sphinx"""
-
     if isinstance(entity, Entity):
         return True
     if isinstance(entity, collections.Mapping):
@@ -553,11 +584,15 @@ def is_versionable(entity):
 
     if isinstance(entity, Versionable):
         return True
-    if 'concreteType' in entity and entity['concreteType'] in _entity_type_to_class:
-        entity_class = _entity_type_to_class[entity['concreteType']]
-    else:
-        entity_class = Entity
-    return issubclass(entity_class, Versionable)
+
+    try:
+        if 'concreteType' in entity and entity['concreteType'] in _entity_type_to_class:
+            entity_class = _entity_type_to_class[entity['concreteType']]
+            return issubclass(entity_class, Versionable)
+    except TypeError:
+        pass
+
+    return False
 
 
 def is_locationable(entity):
